/* cap.c - Linux-specific capability functions
 * Time-stamp: <2004-06-14 njk>
 *  
 * (C) 2004 Nicholas J. Kain <njk@aerifal.cx>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/capability.h>
#include <sys/prctl.h>
#include <pwd.h>
#include <grp.h>

#include <errno.h>

#include "defines.h"
#include "nstrl.h"
#include "log.h"

static void set_cap(uid_t uid, gid_t gid, char *captxt) 
{
    cap_t caps;

    if (!captxt)
		return;
    
    if (prctl(PR_SET_KEEPCAPS, 1)) {
        log_line("FATAL - set_cap: prctl() failed\n");
        exit(EXIT_FAILURE);
    }

    if (setgroups(0, NULL) == -1) {
	    log_line("FATAL - set_cap: setgroups() failed\n");
	    exit(EXIT_FAILURE);
    }

    if (setegid(gid) == -1 || seteuid(uid) == -1) {
	    log_line("FATAL - set_cap: seteuid() failed\n");
	    exit(EXIT_FAILURE);
    }
    
    caps = cap_from_text(captxt);
    if (!caps) {
        log_line("FATAL - set_cap: cap_from_text() failed\n");
        exit(EXIT_FAILURE);
    }
    
    if (cap_set_proc(caps) == -1) {
        log_line("FATAL - set_cap: cap_set_proc() failed\n");
        exit(EXIT_FAILURE);
    }
    
    cap_free(caps);
}

void drop_root(uid_t uid, gid_t gid, char *captxt) 
{
    if (uid == 0 || gid == 0) {
		return;
    }

    if (captxt)
	    set_cap(uid, gid, captxt);

    if (setregid(gid, gid) == -1 || setreuid(uid, uid) == -1) {
        log_line("FATAL - drop_root: failed to drop root!\n");
        exit(EXIT_FAILURE);
    }
}

